<?php
/*
	mobiki - the wiki for the mobile world
	Author:  Martin Doering <martin@datenbrei.de>
	Project: http://mobiki.datenbrei.de
	License: http://mobiki.datenbrei.de/index.php?page=License
*/

	// Set internal character encoding to 'UTF-8' - needed for some functions below
	// Not needed since PHP 5.6 with default_charset = UTF-8
	mb_internal_encoding("UTF-8");

	// load password and site specific constants - MUST be in UTF-8!!!
	require('config.php'); 
	
	// This block of default definitions can be overwritten in config.php

	// Buttons visible in the navigation bar
	defined('HOME_BUTTON') or define('HOME_BUTTON',		'Home');
	defined('SEARCH_BUTTON') or define('SEARCH_BUTTON', '🔎');
	defined('INFO_BUTTON') or define('INFO_BUTTON',		'About');
	defined('NEWS_BUTTON') or define('NEWS_BUTTON', 	'News');
	defined('FEED_BUTTON') or define('FEED_BUTTON', 	'Feed');
	defined('LOGIN_BUTTON') or define('LOGIN_BUTTON', 	'Login');
	defined('LOGOUT_BUTTON') or define('LOGOUT_BUTTON', 'Logout');
	defined('EDIT_BUTTON') or define('EDIT_BUTTON', 	'Edit');
	defined('SAVE_BUTTON') or define('SAVE_BUTTON', 	'Save');

	// Pages linked to the above buttons
	defined('HOME_PAGE') or define('HOME_PAGE',			'Home');
	defined('MENU1') or define('MENU1',					'');
	defined('MENU2') or define('MENU2',					'');
	defined('MENU3') or define('MENU3',					'');
	defined('MENU4') or define('MENU4',					'');
	defined('MENU5') or define('MENU5',					'');
	defined('INFO_PAGE') or define('INFO_PAGE',			'About');
	defined('NEWS_TITLE') or define('NEWS_TITLE', 		'News');
	defined('LOGIN_TITLE') or define('LOGIN_TITLE', 	'Login');
	
	// Descriptions for the above pages
	defined('FEED_DESCRIPTION') or define('FEED_DESCRIPTION', 		'An RSS Feed with recent updates to the site');
	defined('LOGIN_DESCRIPTION') or define('LOGIN_DESCRIPTION', 	'Login for site admins only');
	defined('NEWS_DESCRIPTION') or define('NEWS_DESCRIPTION', 		'An overview over the newest additions');
	defined('HASHTAG_DESCRIPTION') or define('HASHTAG_DESCRIPTION', 'Show pages tagged with the same hashtag');

	// Other texts and messages, output results etc...
	defined('NEW_PAGE') or define('NEW_PAGE', 'New Page');
	defined('SEARCH_RESULTS') or define('SEARCH_RESULTS', 		'Search Results for');
	defined('RESULT_TITLE') or define('RESULT_TITLE', 			'Found in Title');
	defined('RESULT_TEXT') or define('RESULT_TEXT', 			'Found in Text');
	defined('HASHTAG_RESULTS') or define('HASHTAG_RESULTS', 	'Pages tagged with');
	defined('DESC_PLACEHOLDER') or define('DESC_PLACEHOLDER', 	'Write a short pure text description for this page');
	defined('TEXT_PLACEHOLDER') or define('TEXT_PLACEHOLDER', 	'Put here your page content');
	defined('PWD_MESSAGE') or define('PWD_MESSAGE', 			'Type your Password and hit Enter');
	defined('PWD_SET') or define('PWD_SET', 					'Initially set global Password and hit Enter');
	defined('PWD_ERROR') or define('PWD_ERROR', 				'Wrong Password - try again!');

	// set in config.php according to your country and language
	defined('WIKI_LANGUAGE') or define('WIKI_LANGUAGE',			'en');
	defined('LOCALE') or define('LOCALE', 						'en_US.UTF-8');
	defined('TIME_FORMAT') or define('TIME_FORMAT', 			'l, jS \of F Y');
	defined('TIME_FORMAT_LIST') or define('TIME_FORMAT_LIST', 	'jS \of M. Y');

	// Directory settings - normally no need to change
	defined('PAGE_DIR') or define('PAGE_DIR', 					'pages/');
	defined('DESC_DIR') or define('DESC_DIR', 					'descriptions/');
	defined('IMAGE_DIR') or define('IMAGE_DIR', 				'images/');

	// set to "yes" in config.php to make mobiki privat
	defined('LOCKED') or define('LOCKED', 					'no');

	
	// locale and our own path
	@setlocale(LC_ALL, LOCALE);
	define ('MYPATH', ((isset($_SERVER['HTTPS']) && ($_SERVER['HTTPS'] == 'on')) ? 'https://' : 'http://') . $_SERVER['HTTP_HOST'] . $_SERVER['SCRIPT_NAME'] );

	// Regex for mail and http URL validation and internal link
	define('RGX_MAIL', '[0-9a-zA-Z\./~\-_]+@[0-9a-z\./~\-_]+');
	define('RGX_URL', 'https?://[0-9a-zA-Z\.\&amp;\#\:/~\-_%=?+]*');
	define('RGX_IMG', 'image:([0-9a-zA-Z\.\&amp;\#\:/~\-_%=?]*)\.(jpg|gif|png|svg)');
	define('RGX_IMGS', 'images:([0-9a-zA-Z\.\&amp;\#\:/~\-_%=?]*)');
	define('RGX_WIKI', '[^\]]+'); 	// any char except ] 
	define('RGX_RECENT', 'recent:([\d])');
			
	$req = new Request();
	
	// make a sanitized URL from a string
	function html($string) { return htmlspecialchars($string); }
	
	// make a sanitized URL from a string
	function url($string) { return urlencode($string); }
	
	// get filename from pagename
	function filename($pagename) { return preg_replace('/\+/', ' ', urlencode($pagename)) . '.txt'; }

	// get pagename from filename
	function pagename($filename) { return urldecode(mb_substr($filename, 0, mb_strlen(preg_replace('/ /', '+', $filename)) -4)); }
	
	// unique page-id 
	function pageid($pagename) {
		return md5($pagename);
	}
	
	// page url
	function pageurl($pagename) {
		return MYPATH . '?page=' . url($pagename);
	}
	
	// get a page's description from disk
	function description($page) {
		return @file_get_contents(DESC_DIR . filename($page));
	}
		
	function hashtags($string) {  
		$hashtags = '';  
		preg_match_all("/(#\w+)/u", $string, $matches);  
		if ($matches) {
			$hashtagsArray = array_count_values($matches[0]);
			$hashtags = implode(', ', array_keys($hashtagsArray));
		}
		return preg_replace('/#/','',$hashtags);
	}

	class Request {
		public $host;
		public $script;
		public $page; 
		public $cookie; 
		public $query;
		public $action;
		private $message;
		private $password;
		private $content;
		private $description;
		private $domain;
		
		// Here starts the real thing
		function __construct() {
			$this->cookie = (isset($_COOKIE['mobiki'])) ? $_COOKIE['mobiki'] : NULL; 
			$this->host = $_SERVER['HTTP_HOST'];
			$this->domain = ($_SERVER['HTTP_HOST'] != 'localhost') ? $_SERVER['HTTP_HOST'] : FALSE;			
			if ($_SERVER["REQUEST_METHOD"] == "POST") {
				$this->content = (isset($_POST['content'])) ? $_POST['content'] : NULL;
				$this->description = (isset($_POST['description'])) ? $_POST['description'] : NULL;
				$this->password = filter_input(INPUT_POST, "password", FILTER_SANITIZE_STRING,FILTER_FLAG_STRIP_LOW);
				$this->page = htmlspecialchars_decode(filter_input(INPUT_POST, "page", FILTER_UNSAFE_RAW, FILTER_FLAG_STRIP_LOW));
				$this->post();
			}
			elseif ($_SERVER["REQUEST_METHOD"] == "GET") {
				$this->script = $_SERVER['PHP_SELF'];
				$this->page = (isset($_GET['page'])) ? $_GET['page'] : NULL; 
				$this->action = filter_input(INPUT_GET, "action", FILTER_SANITIZE_STRING, FILTER_FLAG_STRIP_LOW);
				$this->message = (filter_input(INPUT_GET, "error", FILTER_VALIDATE_INT)) ? PWD_ERROR : "";
				$this->query = filter_input(INPUT_GET, "query", FILTER_SANITIZE_STRING, FILTER_FLAG_STRIP_LOW);
				$this->get();
			}
		}

		// Write changes to page, if possible and authorized
		function post() {

			// Initial PASSWORD test to enshure no prepared POST hack is possible 
			if (PASSWORD == 'unsetpassword' && $this->page == "" && $this->password != "") {
				$config = file_get_contents('config.php');
				if ($config) {
					$config = str_replace('unsetpassword', crypt($this->password), $config);
					file_put_contents('config.php', $config);
				}
				die(header('Location: index.php'));
			}			

			// Login
			if (crypt($this->password, PASSWORD) == PASSWORD) {
				setcookie('mobiki', PASSWORD, time() + 365*24*3600); // one year
				header("Location: index.php?page=" . url($this->page));
			}

			// If authorized write page and preserve creation time
			elseif ($this->cookie == PASSWORD) {
				$filename = filename($this->page);
				if ($this->description) {
					@file_put_contents(DESC_DIR . $filename, trim($this->description)) or die ('Could not write description to directory "' . DESC_DIR . '" !');
				}
				if ($this->content) {
					$ctime = filemtime(PAGE_DIR . $filename);
					@file_put_contents(PAGE_DIR . $filename, trim($this->content)) or die ('Could not write page to directory "' . PAGE_DIR . '" !');
					touch(PAGE_DIR . $filename, $ctime ? $ctime : time());
					header("Location: index.php?page=" . url($this->page));
				}
				else {
					// saving an empty page means: delete it
					unlink(PAGE_DIR . $filename);
					unlink(DESC_DIR . $filename);
					header('Location: index.php');
				}
				new Sitemap(); // update sitemap.xml
				new Feed(); // update rss feed.xml file
			}
			
			// Password given, but wrong
			else { 
				header("Location: index.php?page=" . url($this->page) . "&action=login&error=1");
			}

		}
		
		function get() {
			$page = new Page();
			
			if (PASSWORD == 'unsetpassword') $page->set_initial_password();
			elseif ($this->action == 'search') $page->print_search_result($this);
			elseif ($this->action == 'hashtag') $page->print_hashtags_found($this);
			elseif ($this->action == 'recent') $page->show_recent_changes($this);
			elseif ($this->action == 'login') $page->show_login_page($this);
			else {
				$page->load_content($this);
				if ($this->action == 'edit') 
					$page->prepare_edit_form($this);
				else
					$page->prepare_html_output($this);
			}

			// drop cookie in browser and actual environment
			if ($this->action == 'logout') {
				setcookie('mobiki', '', time() -3600); 
				$this->cookie = NULL;
			}

			header('Content-Language: ' . WIKI_LANGUAGE);
			header('Cache-Control: no-cache, must-revalidate');

			if (LOCKED == 'yes' && $this->cookie == NULL)
				$page->show_login_page($this);
			$this->print_html($page);
		}

		function print_html($page) {
			// Read template
			$html = file_get_contents('template.html');
			if (! $html) 
				die ("'template.html' is missing!");

			// Put values into template
			if (($this->cookie) && is_writable(PAGE_DIR . filename($this->page)) && ($this->action != 'edit'))
				$html = str_replace('{EDIT}', '<a class="menu" id="edit" href="index.php?page=' . url($this->page) . '&amp;action=edit">' . html(EDIT_BUTTON) . '</a>', $html);
			else
				$html = str_replace('{EDIT}', '', $html);
			
			$html = str_replace('{PAGE_TITLE}', html($page->title), $html);
			$html = str_replace('{PAGE_ID}', pageid($page->title), $html);
			$html = str_replace('{PAGE_URL}', pageurl($page->title), $html);

			$html = str_replace('{TIME}', date(TIME_FORMAT, $page->time), $html);
			$html = str_replace('{ISOTIME}', date('Y-m-d', $page->time), $html);
			$html = str_replace('{CONTENT}', $page->content, $html);
			
			$html = str_replace('{HOST}', $this->host, $html);
			$html = str_replace('{SCRIPT}', $this->script, $html);
			$html = str_replace('{KEYWORDS}', html($page->keywords), $html);
			$html = str_replace('{AUTHOR}', ($this->action) ? 'System' : WIKI_AUTHOR, $html);
			$html = str_replace('{DESCRIPTION}', html($page->description), $html);
			$html = str_replace('{LANGUAGE}', WIKI_LANGUAGE, $html);

			$html = str_replace('{HOME}', '<a class="menu" id="home" title="' . html(WIKI_DESCRIPTION) . '" href="index.php">' . html(HOME_BUTTON) . '</a>', $html);
			$html = str_replace('{WIKI_TITLE}', html(WIKI_TITLE), $html);
			$html = str_replace('{SEARCH}', 	'<form method="get" action="index.php">' . 
												'<input type="hidden" name="action" value="search" />' . 
												'<input id="search" type="search" name="query" value="" placeholder="' . SEARCH_BUTTON . '" />' . 
												'</form>', $html);

			if ($this->action != 'login')
				$html = str_replace('{LOGOUT}', ($this->cookie) ? '<a class="menu" id="logout" href="index.php?page=' . url($this->page) . '&amp;action=logout">' . html(LOGOUT_BUTTON) . '</a>' 
																: '<a class="menu" id="login" title="' . html(LOGIN_DESCRIPTION) . '" href="index.php?page=' . url($this->page) . '&amp;action=login">' . html(LOGIN_BUTTON) . '</a>', $html);
			else
				$html = str_replace('{LOGOUT}', '', $html);
			
			$html = str_replace('{INFO}', '<a class="menu" id="info" title="' . html(description(INFO_PAGE)) . '" href="index.php?page=' . url(INFO_PAGE) . '">' . html(INFO_BUTTON) . '</a>', $html);
			$html = str_replace('{NEWS}', '<a class="menu" id="news" title="' . html(NEWS_DESCRIPTION) . '" href="index.php?action=recent">' . html(NEWS_BUTTON) . '</a>', $html);
			$html = str_replace('{FEED}', '<a class="menu" id="feed" title="' . html(FEED_DESCRIPTION) . '" href="feed.xml">' . html(FEED_BUTTON) . '</a>', $html);
			$html = str_replace('{MESSAGE}', '<span id="message">' . html($this->message) . '</span>', $html);

			$html = str_replace('{MENU1}', MENU1 ? '<a class="menu" id="menu1" title="' . html(description(MENU1)) . '" href="index.php?page=' . url(MENU1) . '">' . html(MENU1) . '</a>' : '', $html);
			$html = str_replace('{MENU2}', MENU2 ? '<a class="menu" id="menu2" title="' . html(description(MENU2)) . '" href="index.php?page=' . url(MENU2) . '">' . html(MENU2) . '</a>' : '', $html);
			$html = str_replace('{MENU3}', MENU3 ? '<a class="menu" id="menu3" title="' . html(description(MENU3)) . '" href="index.php?page=' . url(MENU3) . '">' . html(MENU3) . '</a>' : '', $html);
			$html = str_replace('{MENU4}', MENU4 ? '<a class="menu" id="menu4" title="' . html(description(MENU4)) . '" href="index.php?page=' . url(MENU4) . '">' . html(MENU4) . '</a>' : '', $html);
			$html = str_replace('{MENU5}', MENU5 ? '<a class="menu" id="menu5" title="' . html(description(MENU5)) . '" href="index.php?page=' . url(MENU5) . '">' . html(MENU5) . '</a>' : '', $html);
		
			header('Content-type: text/html; charset=utf-8');
			echo $html;
		}
	}
	
	abstract class Output {

		function getfiles($dirname, $match) {
			$mtime = array();
			$dir = opendir($dirname);
			while ($file = readdir($dir))
				if (!is_dir($file) && preg_match($match, $file))
					$mtime[$file] = filemtime($dirname . $file);
			arsort($mtime);
			return array_slice($mtime, 0, 20);
		}

		// Convert wiki text to HTML
		function wiki_to_html($content) {
			$ref = new Reference();
			$content = preg_replace("/\r\n/Um", "\n", $content);
			$content = preg_replace_callback('#^___+$(.+)^___+$#Ums', array($ref, 'pre'), $content); // save pre text
			$content = preg_replace_callback('#\[(' . RGX_WIKI . ')\|(' . RGX_URL . ')\]#Uu', array($ref, 'link'), $content);  //external link
			$content = preg_replace_callback('#\[(' . RGX_WIKI . ')\|(' . RGX_WIKI . ')\]#Uu', array($ref, 'pwt'), $content);  //internal page with text
			$content = preg_replace_callback('#(' . RGX_URL . '\.(jpg|gif|png|svg))#i', array($ref,'imglink'), $content); // external image
			$content = preg_replace_callback('#' . RGX_IMGS . '#i', array($ref,'images'), $content); // image list
			$content = preg_replace_callback('#' . RGX_IMG . '#i', array($ref,'img'), $content); // local image
			$content = preg_replace_callback('#' . RGX_RECENT . '#i', array($ref,'recent'), $content); // show recently changed pages
			$content = preg_replace_callback('#(' . RGX_URL . ')#i', array($ref, 'url'), $content); //external URL
			$content = preg_replace_callback("/#(\pL+)/u", array($ref, 'hashtag'), $content); // hashtag
			$content = preg_replace_callback("/\[(" . RGX_WIKI . ")\]/Uu", array($ref, 'pagename'), $content); // Internal page
			$content = html($content);
			$content = preg_replace("/^(.+)$/Um", "<p>$1</p>", $content); //at first every line is a paragraph
			$content = preg_replace('/^<p>!(.+)<\/p>$/Um', "<h2>$1</h2>\n", $content);
			$content = preg_replace('#(' . RGX_MAIL . ')#i', '<a class="mail" href="mailto:$0">$0</a>', $content); // mail address URL
			$content = preg_replace('/^<p>---+<\/p>$/Um', "<hr>\n", $content);
			$content = preg_replace('/^<p>[\*\-] *(.+)<\/p>$/Um', "<ul><li>$1</li></ul>", $content);
			$content = preg_replace('/^<p>\d\. *(.+)<\/p>$/Um', "<ol><li>$1</li></ol>", $content);
			$content = preg_replace('/(<\/ol>.<ol>|<\/ul>.<ul>)/Ums', "", $content);
			$content = preg_replace("/'''''(.*)'''''/Um", '<strong><i>$1</i></strong>', $content);
			$content = preg_replace("/'''(.*)'''/Um", '<strong>$1</strong>', $content);
			$content = preg_replace("/''(.*)''/Um", '<i>$1</i>', $content); 
			$content = preg_replace_callback('/{([a-f0-9]{32})}/', array($ref, 'get'), $content); // get back temporarily stored strings
			$content = preg_replace('#^<p>(<img .+/>)</p>$#Um', "$1", $content); // images are no paragraphs
			$content = preg_replace("#</p>\n<p>#Um", "<br>\n", $content); // a single linebreak is not a new paragraph
			return $content;
		}
	
	}
	
	class Page extends Output {
		public $title = "";
		public $time;
		public $content = "";
		public $description = "";
		public $keywords = "";

		function __construct() {
			$this->time = time();
		}

		// Prepare initial password form
		function set_initial_password() {
			$this->title = PWD_SET;
			$this->content = '<form method="post" action="index.php" autocomplete="off">';
			$this->content .= '<input type="password" name="password" autofocus placeholder="' . PWD_SET . '" />';
			$this->content .= '</form>';
		}
	
		// Search with sanatized query
		function print_search_result($req) {
			$this->title = SEARCH_RESULTS . ' "' . $req->query . '"';
			$query = preg_replace('/(\*|\+|\\\\|\/)/u', ".*", $req->query);
			$dir = opendir(PAGE_DIR);
			$result = NULL;
			$files = $this->getfiles(PAGE_DIR, '/.txt/');
			foreach ($files as $filename => $mtime) {
				$pagename= pagename($filename);
				if (preg_match('/' . $query . '/i', $pagename)) {
					$result .= '<a class="page" href="index.php?page=' . url($pagename) . '">' . html($pagename) . '</a>';
					$result .= ' &#8212; ';
					$result .= @file_get_contents(DESC_DIR . $filename) . '<br />';
				}
			}
			if ($result)
				$this->content = '<h2>' . RESULT_TITLE . '</h2><p>' . $result . '</p>';

			$result = NULL;
			$files = $this->getfiles(PAGE_DIR, '/.txt/');
			foreach ($files as $filename => $mtime) {
				if (preg_match('/' . $query . '/i', file_get_contents(PAGE_DIR . $filename))) {
					$pagename = pagename($filename);
					$result .= '<a class="page" href="index.php?page=' . url($pagename) . '">' . html($pagename) . '</a>';
					$result .= ' &#8212; ';
					$result .= @file_get_contents(DESC_DIR . $filename) . '<br />';
				}
			}
			if ($result)
				$this->content .= '<h2>' . RESULT_TEXT . '</h2><p>' . $result . '</p>';
		}

		// Search with sanatized query
		function print_hashtags_found($req) {
			$req->query = "#" . preg_replace('/(\*|\+|\\\\|\/)/', "", $req->query);
			$this->title = HASHTAG_RESULTS . " \"" . $req->query . "\"";
			$this->content = '<p>';
			$files = $this->getfiles(PAGE_DIR, '/.txt/');
			foreach ($files as $filename => $mtime) {
				if (preg_match('/' . $req->query . '/i', file_get_contents(PAGE_DIR . $filename))) {
					$pagename = pagename($filename);
					$this->content .= '<a class="page" href="index.php?page=' . url($pagename) . '">' . html($pagename) . '</a>';
					$this->content .= ' &#8212; ';
					$this->content .= @file_get_contents(DESC_DIR . $filename) . '<br />';
				}
			}
			$this->content .= '</p>';
		}

		// Recent changes
		function show_recent_changes($req) {
			$this->title = NEWS_TITLE;
			$files = $this->getfiles(PAGE_DIR, '/.txt/');
			$this->content = '<p>';
			foreach ($files as $filename => $timestamp) {
				$pagename = pagename($filename);
				$this->content .= '<a class="page" href="index.php?page=' . url($pagename) . '">' . html($pagename) . '</a>' ;
				$this->content .= ' &#8212; ';
				$this->content .= date(TIME_FORMAT_LIST, $timestamp) . '<br />';
				$this->content .= @file_get_contents(DESC_DIR . $filename) . '<br /><br />';
			}
			$this->content .= '</p>';
		}

		// Prepare page formatting
		function load_content($req) {
			if (!$req->page)
				$req->page = HOME_PAGE;
			$this->title = $req->page;
			
			// Read page contents, description and time of last change
			$filename = filename($req->page);
			if (($file = @fopen(PAGE_DIR . $filename, "r")) || $req->action <> "") {
				$this->time = filemtime(PAGE_DIR . $filename);
				$this->content = @fread($file, @filesize(PAGE_DIR . $filename));
				$this->keywords = hashtags($this->content);
				@fclose($file);
				$this->description = @file_get_contents(DESC_DIR . $filename);
			}
			else {
				$req->action = 'edit';
			}
		}	

		// Login page
		function show_login_page($req) {
			$this->title = LOGIN_TITLE;
			$this->content = '<form method="post" action="index.php" autocomplete="off">';
			$this->content .= '<input type="hidden" name="page" value="' . html($req->page) . '" /><br />';
			$this->content .= '<input type="password" name="password" autofocus placeholder="' . PWD_MESSAGE . '" />';
			$this->content .= '</form>';
			$this->message = PWD_MESSAGE;
		}
		
		// Editing the page
		function prepare_edit_form($req) {
			$text = '<form method="post" action="index.php">';
			$text .= '<input type="text" name="description" placeholder="' . DESC_PLACEHOLDER . '" rows="2" value="' . html($this->description) . '"><br />';
			$text .= '<textarea autofocus name="content" placeholder="' . TEXT_PLACEHOLDER . '" rows="16">' . html($this->content) . '</textarea>';
			$text .= '<input type="hidden" name="page" value="' . html($req->page) . '" /><br />';
			$text .= (PASSWORD != "" && $req->cookie == PASSWORD) ? '<input id="save" type="submit" value="' . SAVE_BUTTON . '" />' : '<br />';
			$text .= '</form>';
			$this->content = $text;
		}
		
		// Convert wiki text to HTML
		function prepare_html_output($req) {
			$this->content = $this->wiki_to_html($this->content);
		}


	}

	class Feed extends Output {
		private $xml;
		private $content;
		
		// Generate RSS 2.0 feed
		function __construct() {
			$xml = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
			$xml .= "<rss version=\"2.0\" xmlns:content=\"http://purl.org/rss/1.0/modules/content/\">\n";
			$xml .= "<channel>\n";
			$xml .= "<title>" . html(WIKI_TITLE) . "</title>\n";
			$xml .= "<link>" . MYPATH . "</link>\n";
			$xml .= "<description>" . html(WIKI_DESCRIPTION) . "</description>\n";
			$xml .= "<language>" . WIKI_LANGUAGE . "</language>\n";
			$xml .= "<pubDate>" . html(date(DATE_RSS)) . "</pubDate>\n";
			$xml .= "<generator>mobiki - the wiki for the mobile world</generator>\n";
			$files = $this->getfiles(PAGE_DIR, '/.txt/');
			foreach ($files as $filename => $timestamp) {
				$pagename = pagename($filename);
				$content = @file_get_contents(PAGE_DIR . $filename);
				$description = @file_get_contents(DESC_DIR . $filename);
				$xml .= "<item>\n";
				$xml .= "<title>" . html($pagename) . "</title>\n";
				$xml .= "<link>" . MYPATH . "?page=" . url($pagename) . "</link>\n";
				$xml .= "<description>" . htmlspecialchars($description, ENT_COMPAT | ENT_XML1) . "</description>\n";
				$xml .= "<content:encoded><![CDATA[" . $this->wiki_to_html($content) . "]]></content:encoded>\n";
				$xml .= "<guid>" . pageid($pagename) . "</guid>\n";
				$xml .= "<pubDate>" . html(date(DATE_RSS, $timestamp)) . "</pubDate>\n";
				$xml .= "</item>\n";
			}
			$xml .= "</channel>\n";
			$xml .= "</rss>";
			if (! $file = fopen(getcwd() . '/feed.xml', "w"))
				die ("Could not write feed!");
			fputs($file, $xml);
			fclose($file);
		}
	}

	class Sitemap extends Output {
		private $xml;
		
		// Generate sitemap
		function __construct() {
			$xml = '<?xml version="1.0" encoding="UTF-8"?>' . "\n";
			$xml .= '<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">' . "\n";
			$files = $this->getfiles(PAGE_DIR, '/.txt/');
			foreach ($files as $filename => $mtime) {
				$xml .= "<url>\n";
				$xml .= '<loc>' . MYPATH . '?page=' . url(pagename($filename)) . "</loc>\n";
				$xml .= '<lastmod>' . date('Y-m-d', filemtime(PAGE_DIR . $filename)) . "</lastmod>\n";
				$xml .= "</url>\n";
			}
			$xml .= "</urlset>\n";
			if (! $file = fopen(getcwd() . '/sitemap.xml', "w"))
				die ("Could not write sitemap.xml!");
			fputs($file, $xml);
			fclose($file);
		}
	}
	
	class Reference extends Output {
		private $reference = array();

		// store string and give back a unique reference
		private function put($string) {
			$hash = md5($string);
			$this->reference[$hash] = $string;
			return "{" . $hash . "}";
		}
			
		// take reference and give back the previousely stored string
		function get($m) {
			return $this->reference[$m[1]];
		}
		
		function pre ($m) {return $this->put('<pre>' . html($m[1]) . '</pre>');}
		function link ($m) {return $this->put('<a class="link" title="External Link" href="' . $m[2] . '" target="_blank">' . html($m[1]) . '</a>');}
		function imglink ($m) {return $this->put('<img alt="' . $m[1] . '" title="' . $m[1] . '" src="' . $m[1] . '.' . $m[2] . '" />');}
		function img ($m) {
			$alt = preg_replace('/[-_]/', ' ', $m[1]);
			return $this->put('<img alt="' . $alt . '" title="' . $alt. '" src="images/' . $m[1] . '.' . $m[2] . '" />');
			}
		function url ($m) {return $this->put('<a class="link" title="External Link" href="' . $m[1] . '" target="_blank">' . html($m[1]) . '</a>');}
		function hashtag ($m)  {return $this->put('<a class="hashtag" title="' . html(HASHTAG_DESCRIPTION) . '" href="' . MYPATH . '?action=hashtag&amp;query=' . url($m[1]) . '">' . html($m[1]) . '</a>');}
		function pagename ($m) {return $this->put('<a class="page" title="' . html(description($m[1])) . '" href="' . MYPATH . '?page=' . url($m[1]) . '">' . html($m[1]) . '</a>');}
		function pwt ($m) {return $this->put('<a class="page" title="' . html(description($m[2])) . '" href="' . MYPATH . '?page=' . url($m[2]) . '">' . html($m[1]) . '</a>');}

		// Image List
		function images($m) {
			$html = '';
			$files = $this->getfiles(IMAGE_DIR . $m[1] . '/', "/^.+\.(jpg|gif|png|svg)$/");
			foreach ($files as $file => $mtime) {
				$alt = mb_substr(preg_replace('/[-_]/', ' ', $file), 0, mb_strlen(preg_replace('/ /', '+', $file)) -4);
				$html .= '<img src="images/' . $m[1] . '/' . $file . '" alt="' . $alt . '" title="' . $alt . '" />' . "\n";
			}
			return $this->put($html);
		}

		// List of recent changes, newest to oldest
		function recent($m) {
			//$result = '<p>';
			$count = intval($m[1]);
			if ($count < 1) $count = 1;
			$files = $this->getfiles(PAGE_DIR, '/.txt/');
			foreach ($files as $file => $mtime) {
				if ($count-- <= 0) 
					break;
				$pagename = pagename($file);
				$result .= '<a class="page" title="' . html(description($pagename)) . '" href="index.php?page=' . url($pagename) . '">' . html($pagename) . '</a>' ;
				$result .= ' &#8212; ';
				$result .= date(TIME_FORMAT_LIST, $mtime) . '<br />';
				$result .= @file_get_contents(DESC_DIR . $file) . '<br /><br />';
			}
			//$result .= '</p>';
			return $this->put($result);
		}
	}
?>
